/*
 * Copyright 2008 Perforce Software.  All rights reserved.
 *
 * This file is part of Perforce - the FAST SCM System.
 */

#ifndef MYDTS_HEADER
#define MYDTS_HEADER
/*
 * MyDTS.h is an interface class between P4 DTG and a generic
 * defect tracking system. This class encapsulates the DTS's
 * api.
 *
 */

struct DTGStrList;
struct DTGDate;
#ifdef _WIN32
#include <my_global.h>
#endif
#include <mysql.h>

class MyDTS {
	protected:
	    int valid;
	    int wait_time;
	    int offset;
	    int utf8;
		int repo_repl;
	    int my_port;
	    int ck_privs;
	    char *my_server;
	    char *my_user;
	    char *my_pass;
	    const char *rm_db;
	    const char *rm_cf;

	    int connected( char *&err );
	    int no_privs( char *&err );
	    char *projID;
	    struct DTGStrList *get_options( const char *table, char *&err );
	    struct DTGStrList *get_custom_options( const char *options );
	    struct DTGField *single_row( const char *query, char *&err );
	    struct DTGStrList *single_col( const char *query, char *&err );
	    struct DTGField *two_cols( const char *query );
	    char *get_description( const char *bugid, char *&err );
	    char *get_notes( const char *bugid, char *&err );
	    void replicate_fix( const char *defect, const char *nums, char *&err );

	    MYSQL *mysql;
	    char *use_profile;
	    struct DTGField *field_map;
	    struct DTGField *profile_map;
	    struct DTGField *product_map;
	    struct DTGField *component_map;

	public:
	    MyDTS( const char *server, 
		const char *user, 
		const char *pass,
		const struct DTGField *attrs,
		const char *prog_name,
		const char *prog_ver,
		char *&err);
	    virtual ~MyDTS();

	    int is_valid() { return valid; };
	    int valid_project( const char *proj );
	    int utf8_ok() { return utf8; };
	    int server_offline( struct DTGError *error );

	    void connect_to_project( const char *proj, char *&err );
	    struct DTGStrList *list_projects( char *&err );
	    struct DTGFieldDesc *get_field_desc( const char *defect, 
		char *&err );
	    struct DTGStrList *get_prod_components( const char *product, 
							char *&err );
	    char *get_server_version( char *&err );
	    char *get_server_date( char *&err );
	    struct DTGStrList *list_jobs( int max_rows, 
		struct DTGDate *since, const char *mod_date_field,
		const char *exclude_user, const char *mod_by_field,
		const char *segment_filters, char *&err );
	    struct DTGField *get_defect( const char *defect, char *&err );
	    char *save_defect( const char *defect,
				struct DTGField *fields, char *&err );
	    char* esc_field( const char* fld );
	    struct DTGField *field_names();
	    const char *find_value( struct DTGField *fields, const char *id );
};

#endif
